@@ -5,7 +5,7 @@ Use of this source code is governed by an MIT-style license that can be found in
 """
 
 import dataclasses
-from typing import Optional
+from typing import Optional, List, Dict
 
 import pytest
 
@@ -13,7 +13,7 @@ from tests.dataclasses._helpers import assert_field_default, assert_field_no_def
 from validataclass.dataclasses import validataclass, validataclass_field, Default, DefaultFactory, DefaultUnset, NoDefault
 from validataclass.exceptions import DataclassValidatorFieldException
 from validataclass.helpers import OptionalUnset, UnsetValue
-from validataclass.validators import IntegerValidator, StringValidator, Noneable
+from validataclass.validators import IntegerValidator, StringValidator, Noneable, ListValidator, DictValidator
 
 
 class ValidatorDataclassTest:
@@ -188,6 +188,30 @@ class ValidatorDataclassTest:
         with pytest.raises(TypeError, match="required keyword-only argument"):
             UnitTestDataclass(optional_field=42)
 
+    @staticmethod
+    def test_validataclass_with_mutable_defaults():
+        """
+        Create a dataclass using @validataclass and use mutable values (e.g. lists) as defaults. These are not allowed
+        for regular dataclasses and should use default_factory instead.
+        """
+
+        @validataclass
+        class UnitTestDataclass:
+            field_list: List[int] = (ListValidator(IntegerValidator()), Default([]))
+            field_dict: Dict[str, int] = (DictValidator(field_validators={'foo': IntegerValidator()}), Default({'foo': 0}))
+
+        # Try to instantiate the class using the regular constructor
+        obj1 = UnitTestDataclass()
+        obj2 = UnitTestDataclass()
+
+        # Check values
+        assert obj1.field_list == obj2.field_list == []
+        assert obj1.field_dict == obj2.field_dict == {'foo': 0}
+
+        # Check that the list and dict were deepcopied
+        assert obj1.field_list is not obj2.field_list
+        assert obj1.field_dict is not obj2.field_dict
+
     # Subclassing / inheritance
 
     @staticmethod
@@ -331,6 +355,42 @@ class ValidatorDataclassTest:
         assert fields['non_init'].default == 1
         assert 'validator' not in fields['non_init'].metadata
 
+    @staticmethod
+    def test_validataclass_multiple_inheritance():
+        """ Test that the @validataclass decorator handles multiple inheritance correctly. """
+
+        @validataclass
+        class BaseA:
+            field_a: int = IntegerValidator()
+            field_both: int = (IntegerValidator(), Default(1))
+
+        @validataclass
+        class BaseB:
+            field_b: str = StringValidator()
+            field_both: str = StringValidator()
+
+        @validataclass
+        class SubClass(BaseB, BaseA):
+            # Override the defaults to test that the decorator recognizes all fields of both base classes.
+            # If it does not, a "no validator for field X" error would be raised.
+            field_a: int = Default(42)
+            field_b: Optional[str] = Default(None)
+
+        # Get fields from dataclass
+        fields = get_dataclass_fields(SubClass)
+
+        # Check validators and defaults of overridden fields
+        assert isinstance(fields['field_a'].metadata.get('validator'), IntegerValidator)
+        assert isinstance(fields['field_b'].metadata.get('validator'), StringValidator)
+        assert fields['field_a'].metadata.get('validator_default') == Default(42)
+        assert fields['field_b'].metadata.get('validator_default') == Default(None)
+
+        # For fields defined in both base classes, BaseB should take precedence over BaseA (MRO: SubClass, BaseB, BaseA, object).
+        # Since BaseB does NOT inherit from BaseA, there should NOT be partial overrides in the field, i.e. field_both in
+        # SubClass does not have a default, since the field has no default in BaseB.
+        assert isinstance((fields['field_both'].metadata.get('validator')), StringValidator)
+        assert fields['field_both'].metadata.get('validator_default') is None
+
     # Error cases
 
     @staticmethod
